@@ -152,7 +152,8 @@ check_bad_symbols(Class *c)
 			if((m->method == SIGNAL_LAST_METHOD ||
 			    m->method == SIGNAL_FIRST_METHOD ||
 			    m->method == VIRTUAL_METHOD) &&
-			   (strcmp(m->id, "__parent__")==0 ||
+			   (strcmp(m->id, "_epv")==0 ||
+			    strcmp(m->id, "__parent__")==0 ||
 			    strcmp(m->id, "___parent__")==0)) {
 				error_printf(GOB_ERROR, m->line_no,
 					     "'%s' not allowed as an "
